﻿using System;
using System.ComponentModel;
using System.Data;

namespace TestProject
{
    /// <summary>
    /// 커스텀 속성 설명자
    /// </summary>
    public class CustomPropertyDescriptor : PropertyDescriptor
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 속성 타입
        /// </summary>
        private Type propertyType;

        /// <summary>
        /// 컴포넌트 타입
        /// </summary>
        private Type componentType;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 속성 타입 - PropertyType

        /// <summary>
        /// 속성 타입
        /// </summary>
        public override Type PropertyType
        {
            get
            {
                return this.propertyType;
            }
        }

        #endregion
        #region 컴포넌트 타입 - ComponentType

        /// <summary>
        /// 컴포넌트 타입
        /// </summary>
        public override Type ComponentType
        {
            get
            {
                return this.componentType;
            }
        }

        #endregion
        #region 읽기 전용 여부 - IsReadOnly

        /// <summary>
        /// 읽기 전용 여부
        /// </summary>
        public override bool IsReadOnly
        {
            get
            {
                return false;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 생성자 - CustomPropertyDescriptor(propertyName, attributeArray)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="propertyName">속성명</param>
        /// <param name="attributeArray">특성 배열</param>
        protected CustomPropertyDescriptor(string propertyName, Attribute[] attributeArray) : base(propertyName, attributeArray)
        {
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - CustomPropertyDescriptor(propertyName, propertyType, componentType)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="propertyName">속성명</param>
        /// <param name="propertyType">속성 타입</param>
        /// <param name="componentType">컴포넌트 타입</param>
        public CustomPropertyDescriptor(string propertyName, Type propertyType, Type componentType) : this(propertyName, null)
        {
            this.propertyType  = propertyType;
            this.componentType = componentType;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 값 리셋 가능 여부 구하기 - CanResetValue(component)

        /// <summary>
        /// 값 리셋 가능 여부 구하기
        /// </summary>
        /// <param name="component">컴포넌트</param>
        /// <returns>값 리셋 가능 여부</returns>
        public override bool CanResetValue(object component)
        {
            return false;
        }

        #endregion
        #region 값 리셋하기 - ResetValue(component)

        /// <summary>
        /// 값 리셋하기
        /// </summary>
        /// <param name="component">컴포넌트</param>
        public override void ResetValue(object component)
        {
            throw new NotImplementedException();
        }

        #endregion
        #region 값 구하기 - GetValue(component)

        /// <summary>
        /// 값 구하기
        /// </summary>
        /// <param name="component">컴포넌트</param>
        /// <returns>값</returns>
        public override object GetValue(object component)
        {
            DataRow dataRow = (DataRow)component;

            return dataRow[Name];
        }

        #endregion
        #region 값 설정하기 - SetValue(component, value)

        /// <summary>
        /// 값 설정하기
        /// </summary>
        /// <param name="component">컴포넌트</param>
        /// <param name="value">값</param>
        public override void SetValue(object component, object value)
        {
            DataRow dataRow = (DataRow)component;

            dataRow[Name] = value;
        }

        #endregion
        #region 값 직렬화 여부 구하기 - ShouldSerializeValue(component)

        /// <summary>
        /// 값 직렬화 여부 구하기
        /// </summary>
        /// <param name="component">컴포넌트</param>
        /// <returns>값 직렬화 여부</returns>
        public override bool ShouldSerializeValue(object component)
        {
            return true;
        }

        #endregion
    }
}