﻿using System.ComponentModel;
using System.Data;

namespace TestProject
{
    /// <summary>
    /// 데이터 행 호스
    /// </summary>
    public class DataRowSource : BindingList<DataRow>, ITypedList
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 데이터 테이블
        /// </summary>
        private readonly DataTable dataTable;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - DataRowSource(dataTable)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="dataTable">데이터 테이블</param>
        public DataRowSource(DataTable dataTable)
        {
            this.dataTable = dataTable;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 항목 속성 컬렉션 구하기 - GetItemProperties(sourceArray)

        /// <summary>
        /// 항목 속성 컬렉션 구하기
        /// </summary>
        /// <param name="sourceArray">소스 배열</param>
        /// <returns>항목 속성 컬렉션</returns>
        public PropertyDescriptorCollection GetItemProperties(PropertyDescriptor[] sourceArray)
        {
            if(this.dataTable == null)
            {
                return null;
            }

            PropertyDescriptor[] targetArray = new CustomPropertyDescriptor[this.dataTable.Columns.Count];

            for(int i = 0; i < targetArray.Length; i++)
            {
                targetArray[i] = new CustomPropertyDescriptor
                (
                    this.dataTable.Columns[i].ColumnName,
                    this.dataTable.Columns[i].DataType,
                    typeof(DataRow)
                );
            }

            PropertyDescriptorCollection targetCollection = new PropertyDescriptorCollection(targetArray);

            return targetCollection;
        }

        #endregion
        #region 리스트명 구하기 - GetListName(sourceArray)

        /// <summary>
        /// 리스트명 구하기
        /// </summary>
        /// <param name="sourceArray">소스 배열</param>
        /// <returns>리스트명</returns>
        public string GetListName(PropertyDescriptor[] sourceArray)
        {
            return typeof(DataRow).Name;
        }

        #endregion
    }
}