﻿using System;
using System.Data;
using System.Threading.Tasks;

using DevExpress.Data;
using DevExpress.Utils;
using DevExpress.XtraEditors;
using DevExpress.XtraGrid.Views.Grid;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 배치 크기
        /// </summary>
        private int batchSize = 10;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.gridView.InitializeView
            (
                true,                           // Multi Select
                GridMultiSelectMode.CellSelect, // Grid Multi Select Mode
                DrawFocusRectStyle.CellFocus,   // Draw Focus Rect Style
                true,                           // Show Indicator
                true,                           // Show Column Headers
                true,                           // Allow Column Moving
                true,                           // Allow Column Resizing
                true,                           // Allow Filter
                true,                           // Allow Sort
                false,                          // Allow Cell Merge
                EditorShowMode.Default,         // Editor Show Mode
                false                           // Editable
            );

            this.gridView.OptionsBehavior.AutoPopulateColumns = true;

            VirtualServerModeSource virtualServerModeSource = new VirtualServerModeSource();

            virtualServerModeSource.MoreRows         += virtualServerModeSource_MoreRows;
            virtualServerModeSource.AcquireInnerList += virtualServerModeSource_AcquireInnerList;

            this.gridControl.DataSource = virtualServerModeSource;

            this.gridView.BestFitColumns();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 가상 서버 모드 소스 내부 리스트 획득시 처리하기 - virtualServerModeSource_AcquireInnerList(sender, e)

        /// <summary>
        /// 가상 서버 모드 소스 내부 리스트 획득시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void virtualServerModeSource_AcquireInnerList(object sender, VirtualServerModeAcquireInnerListEventArgs e)
        {
            DataTable sourceDataTable = GetDataTable(0, 0);

            e.InnerList = new DataRowSource(sourceDataTable);
        }

        #endregion
        #region 가상 서버 모드 소스 추가 행 처리하기 - virtualServerModeSource_MoreRows(sender, e)

        /// <summary>
        /// 가상 서버 모드 소스 추가 행 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void virtualServerModeSource_MoreRows(object sender, VirtualServerModeRowsEventArgs e)
        {
            e.RowsTask = Task.Factory.StartNew
            (
                () =>
                {
                    DataTable batchDataTable = GetDataTable(e.CurrentRowCount, batchSize);

                    DataRowSource dataRowSource = new DataRowSource(batchDataTable);

                    for(int i = 0; i < batchDataTable.Rows.Count; i++)
                    {
                        dataRowSource.Add(batchDataTable.Rows[i]);
                    }

                    return new VirtualServerModeRowsTaskResult(dataRowSource, true);
                },
                e.CancellationToken
            );
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 데이터 테이블 구하기 - GetDataTable(startID, count)

        /// <summary>
        /// 데이터 테이블 구하기
        /// </summary>
        /// <param name="startID">시작 ID</param>
        /// <param name="count">카운트</param>
        /// <returns>데이터 테이블</returns>
        private DataTable GetDataTable(int startID, int count)
        {
            DataTable dataTable = new DataTable();

            dataTable.Columns.Add("ID"              , typeof(int     ));
            dataTable.Columns.Add("Name"            , typeof(string  ));
            dataTable.Columns.Add("IsActive"        , typeof(bool    ));
            dataTable.Columns.Add("OrderCount"      , typeof(int     ));
            dataTable.Columns.Add("RegistrationDate", typeof(DateTime));

            for(int i = startID; i < startID + count; i++)
            {
                dataTable.Rows.Add(i, "Name" + i, i % 2 == 0, i * 10, DateTime.Now.AddDays(i));
            }

            return dataTable;
        }

        #endregion
    }
}